import java.util.Iterator;
import java.util.NoSuchElementException;

public class LinkedList<E> implements Iterable<E> {

	private ListNode head;
	private ListNode tail;
	private int size;

	public LinkedList() {
		head = new ListNode(null);
		tail = new ListNode(null);
		head.nextNode = tail;
		tail.prevNode = head;
		size = 0;
	}

	public boolean add(E e) {

		ListNode newNode = new ListNode(e);

		// add links to newNode
		newNode.nextNode = tail;
		newNode.prevNode = tail.prevNode;

		// modify links in the list
		tail.prevNode = newNode;
		newNode.prevNode.nextNode = newNode;

		size++;

		return true;
	}

	public void add(int index, E e) {

		if (index < 0 || index > size) {
			throw new IndexOutOfBoundsException();
		}

		ListNode newNode = new ListNode(e);

		ListNode curNode = head;
		for (int i = 0; i < index; i++) {
			curNode = curNode.nextNode;
		}
		// curNode is at index-1
		// add links to newNode
		newNode.nextNode = curNode.nextNode;
		newNode.prevNode = curNode;

		// redirect the links for our list
		curNode.nextNode.prevNode = newNode;
		curNode.nextNode = newNode;

		size++;
	}

	public E remove() {
		return remove(0);
	}

	public E remove(int index) {

		if (index < 0 || index >= size) 
		{
			throw new IndexOutOfBoundsException();
		}

		E toReturn;
		ListNode curNode = head;
        for (int i = -1; i < index; i++) 
        {
            curNode = curNode.nextNode;
        }
        // curNode is at index
        toReturn = curNode.value;
        curNode.prevNode.nextNode = curNode.nextNode;
        curNode.nextNode.prevNode = curNode.prevNode;

		size--;

		return toReturn;
	}

	public boolean remove(E e) {

		ListNode curNode = head.nextNode;
		while (curNode != tail && !curNode.value.equals(e)) {
			curNode = curNode.nextNode;
		}

		// curNode is just before the node we want to remove
		// curNode is the last node --> e does not exist in the list
		if (curNode == tail) {
			return false;
		} else {
			curNode.nextNode.prevNode = curNode.prevNode;
			curNode.prevNode.nextNode = curNode.nextNode;
			size--;
			return true;
		}
	}

	public void print()
	{
		System.out.println("\nList contents:");
		ListNode nodePtr = head.nextNode;
		while(nodePtr != tail)
		{
			System.out.println(nodePtr.value);
			nodePtr = nodePtr.nextNode;
		}
		System.out.println();
	}
	

	@Override
	public Iterator<E> iterator() {
		return new ListIterator();
	}

	private class ListIterator implements Iterator<E> {

		private ListNode curNode;

		public ListIterator() {
			curNode = head;
		}

		@Override
		public boolean hasNext() {
			return curNode.nextNode != tail;
		}

		@Override
		public E next() {
			if (!hasNext()) {
				throw new NoSuchElementException();
			}

			curNode = curNode.nextNode;
			return curNode.value;
		}

	}


	private class ListNode {

		private E value;
		private ListNode prevNode;
		private ListNode nextNode;

		public ListNode(E e) {
			value = e;
			prevNode = null;
			nextNode = null;
		}

		public void setNext(ListNode next) {
			nextNode = next;
		}

		public ListNode getNext() {
			return nextNode;
		}

		public void setPrev(ListNode prev) {
			prevNode = prev;
		}

		public ListNode getPrev() {
			return prevNode;
		}

		public E getValue() {
			return value;
		}
	}

}
